]> Shamusworld >> Repos - rmac/blobdiff - rmac.c
Version bump for last patch; now at v1.13.4.
[rmac] / rmac.c
diff --git a/rmac.c b/rmac.c
index 77c37e710b4099291a02eef51dc61a1976be9854..9dd75fdb86f8efabe6f92c990fd6a278c4e4c3da 100644 (file)
--- a/rmac.c
+++ b/rmac.c
@@ -1,7 +1,7 @@
 //
-// RMAC - Reboot's Macro Assembler for the Atari Jaguar Console System
+// RMAC - Reboot's Macro Assembler for all Atari computers
 // RMAC.C - Main Application Code
-// Copyright (C) 199x Landon Dyer, 2011 - 2016 Reboot and Friends
+// Copyright (C) 199x Landon Dyer, 2011-2018 Reboot and Friends
 // RMAC derived from MADMAC v1.07 Written by Landon Dyer, 1986
 // Source utilised with the kind permission of Landon Dyer
 //
@@ -25,6 +25,7 @@
 
 int perm_verb_flag;                            // Permanently verbose, interactive mode
 int list_flag;                                 // "-l" listing flag on command line
+int list_pag = 1;                              // Enable listing pagination by default
 int verb_flag;                                 // Be verbose about what's going on
 int m6502;                                             // 1, assembling 6502 code
 int as68_flag;                                 // as68 kludge mode
@@ -32,12 +33,15 @@ int glob_flag;                                      // Assume undefined symbols are global
 int lsym_flag;                                 // Include local symbols in object file
 int sbra_flag;                                 // Warn about possible short branches
 int prg_flag;                                  // !=0, produce .PRG executable (2=symbols)
+int prg_extend;                                        // !=0, output extended .PRG symbols
 int legacy_flag;                               // Do stuff like insert code in RISC assembler
 int obj_format;                                        // Object format flag
 int debug;                                             // [1..9] Enable debugging levels
 int err_flag;                                  // '-e' specified
 int err_fd;                                            // File to write error messages to
 int rgpu, rdsp;                                        // Assembling Jaguar GPU or DSP code
+int robjproc;                                  // Assembling Jaguar Object Processor code
+int dsp56001;                                  // Assembling DSP 56001 code
 int list_fd;                                   // File to write listing to
 int regbank;                                   // RISC register bank
 int segpadsize;                                        // Segment padding size
@@ -47,7 +51,20 @@ char * firstfname;                           // First source filename
 char * cmdlnexec;                              // Executable name, pointer to ARGV[0]
 char * searchpath;                             // Search path for include files
 char defname[] = "noname.o";   // Default output filename
-int optim_flags[OPT_COUNT];    // Specific optimisations on/off matrix
+int optim_flags[OPT_COUNT];            // Specific optimisations on/off matrix
+int activecpu = CPU_68000;             // Active 68k CPU (68000 by default)
+int activefpu = FPU_NONE;              // Active FPU (none by default)
+
+
+//
+// Convert a string to uppercase
+//
+void strtoupper(char * s)
+{
+       while (*s)
+               *s++ &= 0xDF;
+}
+
 
 //
 // Manipulate file extension.
@@ -125,7 +142,7 @@ void DisplayHelp(void)
                "\n"
                "Options:\n"
                "  -? or -h          Display usage information\n"
-               "  -dsymbol[=value]  Define symbol\n"
+               "  -dsymbol[=value]  Define symbol (with optional value, default=0)\n"
                "  -e[errorfile]     Send error messages to file, not stdout\n"
                "  -f[format]        Output object file format\n"
                "                    a: ALCYON (use this for ST)\n"
@@ -134,19 +151,36 @@ void DisplayHelp(void)
                "                    x: com/exe/xex (Atari 800)\n"
                "  -i[path]          Directory to search for include files\n"
                "  -l[filename]      Create an output listing file\n"
+               "  -l*[filename]     Create an output listing file without pagination\n"
+               "  -m[cpu]           Select default CPU. Available options:\n"
+               "                    68000\n"
+               "                    68020\n"
+               "                    68030\n"
+               "                    68040\n"
+               "                    68060\n"
+               "                    6502\n"
+               "                    tom\n"
+               "                    jerry\n"
                "  -n                Don't do things behind your back in RISC assembler\n"
                "  -o file           Output file name\n"
                "  +o[value]         Turn a specific optimisation on\n"
                "                    Available optimisation values and default settings:\n"
-               "                    o0: Absolute long adddresses to word (on)\n"
-               "                    o1: move.l #x,dn/an to moveq         (on)\n"
-               "                    o2: Word branches to short           (on)\n"
-               "                    o3: Outer displacement 0(an) to (an) (off)\n"
+               "                    o0: Absolute long adddresses to word        (on)\n"
+               "                    o1: move.l #x,dn/an to moveq                (on)\n"
+               "                    o2: Word branches to short                  (on)\n"
+               "                    o3: Outer displacement 0(an) to (an)        (off)\n"
+               "                    o4: lea size(An),An to addq #size,An        (off)\n"
+               "                    o5: Absolute long base displacement to word (off)\n"
+               "                    o6: Null branches to NOP                    (off)\n"
+               "                    o7: clr.l Dx to moveq #0,Dx                 (off)\n"
+               "                    o8: adda.w/l #x,Dy to addq.w/l #x,Dy        (off)\n"
+               "                    o9: adda.w/l #x,Dy to lea x(Dy),Dy          (off)\n"
                "  ~o[value]         Turn a specific optimisation off\n"
                "  +oall             Turn all optimisations on\n"
                "  ~oall             Turn all optimisations off\n"
                "  -p                Create an ST .prg (without symbols)\n"
                "  -ps               Create an ST .prg (with symbols)\n"
+               "  -px               Create an ST .prg (with exsymbols)\n"
                "                    Forces -fa\n"
                "  -r[size]          Pad segments to boundary size specified\n"
                "                    w: word (2 bytes, default alignment)\n"
@@ -175,7 +209,7 @@ void DisplayVersion(void)
                "| |  | | | | | | (_| | (__ \n"
                "|_|  |_| |_| |_|\\__,_|\\___|\n"
                "\nReboot's Macro Assembler\n"
-               "Copyright (C) 199x Landon Dyer, 2011-2017 Reboot\n"
+               "Copyright (C) 199x Landon Dyer, 2011-2018 Reboot\n"
                "V%01i.%01i.%01i %s (%s)\n\n", MAJOR, MINOR, PATCH, __DATE__, PLATFORM);
 }
 
@@ -248,12 +282,13 @@ int Process(int argc, char ** argv)
        err_flag = 0;                                   // Initialise error flag
        rgpu = 0;                                               // Initialise GPU assembly flag
        rdsp = 0;                                               // Initialise DSP assembly flag
+       robjproc = 0;                                   // Initialise OP assembly flag
        lsym_flag = 1;                                  // Include local symbols in object file
        regbank = BANK_N;                               // No RISC register bank specified
        orgactive = 0;                                  // Not in RISC org section
        orgwarning = 0;                                 // No ORG warning issued
        segpadsize = 2;                                 // Initialise segment padding size
-       m6502 = 0;                      // 6502 mode off by default
+       m6502 = 0;                                              // 6502 mode off by default
 
        // Initialise modules
        InitSymbolTable();                              // Symbol table
@@ -300,7 +335,7 @@ int Process(int argc, char ** argv)
                                }
 
                                sy->sattr = DEFINED | EQUATED | ABS;
-                               sy->svalue = (*s ? (VALUE)atoi(s) : 0);
+                               sy->svalue = (*s ? (uint64_t)atoi(s) : 0);
                                break;
                        case 'e':                               // Redirect error message output
                        case 'E':
@@ -343,11 +378,51 @@ int Process(int argc, char ** argv)
                                break;
                        case 'l':                               // Produce listing file
                        case 'L':
-                               list_fname = argv[argno] + 2;
+                               if (*(argv[argno] + 2) == '*')
+                               {
+                                       list_fname = argv[argno] + 3;
+                                       list_pag = 0;    // Special case - turn off pagination
+                               }
+                               else
+                               {
+                                       list_fname = argv[argno] + 2;
+                               }
+
                                listing = 1;
                                list_flag = 1;
                                lnsave++;
                                break;
+                       case 'm':
+                       case 'M':
+                               if (strcmp(argv[argno] + 2, "68000") == 0)
+                                       d_68000();
+                               else if (strcmp(argv[argno] + 2, "68020") == 0)
+                                       d_68020();
+                               else if (strcmp(argv[argno] + 2, "68030") == 0)
+                                       d_68030();
+                               else if (strcmp(argv[argno] + 2, "68040") == 0)
+                                       d_68040();
+                               else if (strcmp(argv[argno] + 2, "68060") == 0)
+                                       d_68060();
+                               else if (strcmp(argv[argno] + 2, "68881") == 0)
+                                       d_68881();
+                               else if (strcmp(argv[argno] + 2, "68882") == 0)
+                                       d_68882();
+                               else if (strcmp(argv[argno] + 2, "56001") == 0)
+                                       d_56001();
+                               else if (strcmp(argv[argno] + 2, "6502") == 0)
+                                       d_6502();
+                               else if (strcmp(argv[argno] + 2, "tom") == 0)
+                                       d_gpu();
+                               else if (strcmp(argv[argno] + 2, "jerry") == 0)
+                                       d_dsp();
+                               else
+                               {
+                                       printf("Unrecognized CPU '%s'\n", argv[argno] + 2);
+                                       errcnt++;
+                                       return errcnt;
+                               }
+                               break;
                        case 'o':                               // Direct object file output
                        case 'O':
                                if (argv[argno][2] != EOS)
@@ -369,7 +444,8 @@ int Process(int argc, char ** argv)
                        case 'P':
                                /*
                                 * -p           .PRG generation w/o symbols
* -ps  .PRG generation with symbols
                                 */
                                switch (argv[argno][2])
                                {
@@ -382,9 +458,14 @@ int Process(int argc, char ** argv)
                                                prg_flag = 2;
                                                break;
 
+                                       case 'x':
+                                       case 'X':
+                                               prg_flag = 3;
+                                               break;
+
                                        default:
                                                printf("-p: syntax error\n");
-                                               ++errcnt;
+                                               errcnt++;
                                                return errcnt;
                                }
 
@@ -432,7 +513,7 @@ int Process(int argc, char ** argv)
                                if (pagelen < 10)
                                {
                                        printf("-y: bad page length\n");
-                                       ++errcnt;
+                                       errcnt++;
                                        return errcnt;
                                }
 
@@ -611,15 +692,13 @@ int main(int argc, char ** argv)
        perm_verb_flag = 0;                             // Clobber "permanent" verbose flag
        legacy_flag = 1;                                // Default is legacy mode on (:-P)
 
-       // Set legacy optimisation flags to on
-       // and everything else to off
+       // Set legacy optimisation flags to on and everything else to off
        memset(optim_flags, 0, OPT_COUNT * sizeof(int));
        optim_flags[OPT_ABS_SHORT] =
                optim_flags[OPT_MOVEL_MOVEQ] =
                optim_flags[OPT_BSR_BCC_S] = 1;
 
        cmdlnexec = argv[0];                    // Obtain executable name
-
        endian = GetEndianess();                // Get processor endianess
 
        // If commands were passed in, process them
@@ -369,7 +444,8 @@ int Process(int argc, char ** argv)
                                /*
                                 * -p           .PRG generation w/o symbols
-                                * -ps  .PRG generation with symbols
+                                * -ps          .PRG generation with symbols
+                                * -px          .PRG generation with extended symbols
                                 */
                                switch (argv[argno][2])